#!/usr/env perl

use Test::More tests => 1;

BEGIN {
    use_ok( 'Catalyst::TraitFor::Controller::reCAPTCHA' );
}

#diag( "Testing Catalyst::TraitFor::Controller::reCAPTCHA $Catalyst::TraitFor::Controller::reCAPTCHA::VERSION, Perl $], $^X" );
